package reconciler

import (
	"context"
	"sort"

	"cloud.google.com/go/logging"
	"github.com/presslabs/controller-util/syncer"
	interfaces "k8s.libre.sh/interfaces"
	"k8s.libre.sh/objects"
	"k8s.libre.sh/status"
	ctrl "sigs.k8s.io/controller-runtime"
	"sigs.k8s.io/controller-runtime/pkg/reconcile"
)

type ResourceReconciler struct {
	*ReconcilerBase
	Object objects.Object
	Owner  interfaces.Object
	Status status.ObjectStatus
	log    logging.Logger
	ctx    context.Context
}

func NewResourceReconciler(r *ReconcilerBase, o objects.Object, owner interfaces.Object, ctx context.Context) *ResourceReconciler {
	return &ResourceReconciler{
		ReconcilerBase: r,
		Object:         o,
		Owner:          owner,
		ctx:            ctx,
	}
}

func (r *ResourceReconciler) Reconcile(request reconcile.Request) (reconcile.Result, error) {

	/* 	s := objects.NewObjectSyncer(r.Object, r.Owner, r.ReconcilerBase)

	   	syncResult, err := s.Sync(context.TODO())
	   	if err != nil {
	   		Record(s.GetOwner(), r.GetRecorder(), syncResult)
	   		return ctrl.Result{}, err
	   	}
	   	obj := s.GetObject().(interfaces.Object)

	   	err = WaitUntilCompletion(r.ReconcilerBase, obj)
	   	if err != nil {
	   		Record(r.Owner, r.GetRecorder(), syncResult)
	   		return ctrl.Result{}, err
	   	}

	   	r.Status.SetFromObject(obj)

	   	// TODO TOFIX
	   	r.Status.Status = "Ready"

	   	// TODO UPDATE STATUS of owner ??

	   	Record(r.Owner.DeepCopyObject(), r.GetRecorder(), syncResult) */

	return ctrl.Result{}, nil
}

func SyncObject(ctx context.Context, r interfaces.Reconcile, syncer syncer.Interface) (status.ObjectStatus, error) {

	resource := status.ObjectStatus{}
	syncResult, err := syncer.Sync(ctx)
	obj := syncer.GetObject().(interfaces.Object)

	if err != nil {
		Record(syncer.GetOwner(), r.GetRecorder(), syncResult)
		//	resource.Status = "NotReady"
		return resource, err
	}

	err = WaitUntilCompletion(r, obj)
	if err != nil {
		Record(syncer.GetOwner(), r.GetRecorder(), syncResult)
		return resource, err
	}

	// TO DO TO fix gvk is not recognized, need to get resource with client before
	resource = status.ObjectToStatus(obj)

	// TODO TOFIX
	resource.Status = "Ready"

	Record(syncer.GetOwner(), r.GetRecorder(), syncResult)

	return resource, nil
}

func SyncObjects(ctx context.Context, r interfaces.Reconcile, syncers map[int]syncer.Interface) (resources status.Resources, err error) {
	keys := []int{}

	for k := range syncers {
		keys = append(keys, k)
	}

	sort.Ints(keys)

	for _, k := range keys {
		objStatus, err := SyncObject(ctx, r, syncers[k])
		if err != nil {
			return resources, err
		}
		resources.Objects = append(resources.Objects, objStatus)
	}

	return resources, nil
}
